#ifndef DATADIRECTORY_H
#define DATADIRECTORY_H

#include "datafile.h"

class DataDirectory : public DataFile
{
    Q_OBJECT
public:
    explicit DataDirectory(QString path, QObject *parent = nullptr, bool fromDisk = false);
    ~DataDirectory();
public:
    DataFile *at(int);
    int of(DataFile *);
    int of(DataDirectory *);
    bool take(DataFile *);
    void append(DataFile *d);
    void filterAppend(DataFile *d) { m_oChildren << d; }
    int count() const;
    int counts() const { return m_oChildren.size(); }
    unsigned long total() const { return m_u64Dirs + m_u64Files; }
    unsigned long nSize() const { return m_u64NewDataSize; }
    bool isRoot() const { return m_bIsRoot; }
    bool canClean() const;
    bool duplicate(QString) const;
    void rename(QString, bool add = false);
    void updateChild(QString, QString);
    void clear();
    void clean();
    void copy();
    void *findChildByDisplay(QString, bool child = false);
    bool filterHidden() const { return m_bFilterHidden; }
    bool filterBrokenSymbolLinks() const { return m_bFilterBroken; }
    bool repleaceSymbolLinks() const { return m_bRepleace; }
    void execHidden(bool b = true);
    void execBroken(bool b = true);
    void execRepleace(bool b = true);
    void turnViewFilterFileToHidden();
    void turnViewFilterFileToBrokenSymbolLinks();
    void turnViewFilterFileToRepleaceSymbolLinks();
public slots:
    void doSize(unsigned long, bool);
    void doDirectory(bool, DataFile *);
    void doFile(bool, DataFile *);
    void doChild(DataFile *, bool);
private:
    bool                        m_bIsRoot;
    bool                        m_bFilterHidden;
    bool                        m_bFilterBroken;
    bool                        m_bRepleace;
    unsigned long               m_u64Dirs;
    unsigned long               m_u64Files;
    unsigned long               m_u64NewDataSize;
    QList<DataFile *>           m_oChildren;
    QHash<QString, DataFile *>  m_oHashChildren;
signals:
    void updateSize(unsigned long);
};

#endif // DATADIRECTORY_H
