#ifndef DATAFILE_H
#define DATAFILE_H

#include <QObject>
#include <QIcon>

#define NAME_LENGTH 103

class DataFile : public QObject
{
    Q_OBJECT
public:
    explicit DataFile(QString path, QObject *parent = nullptr, bool fromDisk = false);
    ~DataFile();
public:
    bool isDirectory() const { return m_bIsDirectory; }
    bool isDisk() const { return m_bIsDisk; }
    QIcon   icon() const { return m_oIcon; }
    unsigned long size() const { return m_u64Size; }
    QString stringSize();
    QString display() const { return m_oDisplayName; }
    QString source() const { return m_oSource; }
    QString destination() const { return m_oDestination; }
    DataFile *linkTarget() { return m_oLinkTarget; }
    void setLinkTarget(DataFile *t) { m_oLinkTarget = t; }
    QString type() const { return m_oType; }
    unsigned int level() const { return m_u32Level; }
    bool attrHidden() const { return m_bAttrHidden; }
    bool attrSymbolLink() const { return m_bAttrLink; }
    void rename(QString, bool add = false);
    void copy();
    void filter();
    void changeParent(DataFile *p);
    bool valid() const { return m_bIsValid; }
    void setValid(bool b) { m_bIsValid = b; }
public slots:
    void onUpdate(QString);
private slots:
    void slotsDoHidde(bool);
    void slotsDoBroken(bool);
    void slotsDoRepleace(bool);
protected:
    unsigned int     m_u32Level;
    bool             m_bIsDirectory;
    bool             m_bAttrHidden;
    bool             m_bAttrLink;
    bool             m_bIsDisk;
    bool             m_bIsValid;
    unsigned long    m_u64Size;
    QString          m_oRealName;
    QString          m_oDisplayName;
    QString          m_oSource;
    QString          m_oDestination;
    QString          m_oType;
    QString          m_oCreateTime;
    QString          m_oAddTime;
    DataFile        *m_oLinkTarget;
    QIcon            m_oIcon;
signals:
    // log
    void info(QString); void warn(QString);
    void error(QString); void debug(QString);
    // logic
    void size(unsigned long, bool);
    void file(bool, DataFile *f);
    void directory(bool, DataFile *d);
    // child
    void child(DataFile *, bool);
    // copy
    void copied(QString, /* to */QString);
    void update(QString);
    // filter
    void hidden(bool b = true);
    void broken(bool b = true);
    void repleace(bool b = true);
    void doHidden(void *);
    void doBroken(void *);
    void doRepleace(void *);
};

#endif // DATAFILE_H
