#include "processburndata.h"
#include "src/frame/logger.h"
#include "src/po/translation.h"
#include "src/data/datacdrom.h"
#include "src/data/datadirectory.h"
#include "src/view/modaldialog.h"
#include "src/view/execdialog.h"

#include <QFileInfo>
#include <QDir>
#include <QCoreApplication>
#include <QStandardPaths>
#include <fstream>

ProcessBurnData::ProcessBurnData(void *root, void *cdrom, QObject *parent) :
    ProcessBasic(parent)
{
    DataCDROM *c;
    m_oCDROM = cdrom;
    m_oRoot = root;
    c = (DataCDROM *)cdrom;
    m_fSpeed = 0;
    if (c->mediumStatus() == DataMedium::MEDIUM_EMPTY)
    {
        LOGGER->registerName(this, _("PROCESS_BD"));
        title(_("Burning Data"));
        m_oProcessName = _("Burn Data");
        m_bAppendBurn = false;
    }
    else
    {
        LOGGER->registerName(this, _("PROCESS_AB"));
        title(_("Append Burning"));
        m_oProcessName = _("Append Burn");
        m_bAppendBurn = true;
    }
    MODAL->show();
    setRange(0, 100);
    setValue(0);
}

bool ProcessBurnData::prepare()
{
    DataCDROM *cdrom;
    QFileInfo f;
    ifstream fin;
    std::string s;
    QString sysid;

    m_oISOOutput.clear();
    m_oProcess->setProgram("/bin/bash");
    cdrom = (DataCDROM *)m_oCDROM;

    m_iProgress = 0;
    fin.open("/etc/kylin-build");
    if (fin.is_open())
    {
        std::getline(fin, s);
        sysid = QString::fromStdString(s);
    }
    if (sysid.isEmpty()) sysid = _("KylinOS");

    if (cdrom->mediumType().startsWith("CD"))
        f.setFile("/bin/cdrecord");
    else
        f.setFile("/bin/growisofs");
    if (!f.exists() || !f.isExecutable())
    {
        error(_("There is no %1 program or %1 cannot execute."
                "Please check the cdrecord and growisofs is "
                "exists or not, and give the right permissions"
                ".%2 exiting...")
              .arg(f.absoluteFilePath())
              .arg("KylinBurner"));
        return false;
    }
    connect(this, SIGNAL(cancel(bool)), MODAL, SLOT(cancelEnable(bool)));
    m_bCreateISO = cdrom->settingCreateISO();
    if (m_bCreateISO)
    {
        f.setFile("/bin/mkisofs");
        if (!f.exists() || !f.isExecutable())
        {
            error(_("There is no %1 program or %1 cannot execute."
                    "Please check the mkisofs is exists or not, and give "
                    "the right permissions.%2 exiting...")
                  .arg(f.absoluteFilePath())
                  .arg("KylinBurner"));
            return false;
        }
        m_oISOOutput = QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);
        m_oISOOutput.append("/").append(cdrom->settingVolumnID()).append(".iso");
        f.setFile(m_oISOOutput);
        if (f.exists())
        {
            if (!EXEC->question(_("Create ISO") + " - " + _("Already Exists"),
                               _("There is already have the iso file named %1,"
                                 "replace it or not?").arg(f.fileName())))
                return false;
            if (f.isFile()) QFile::remove(f.absoluteFilePath());
            else { QDir dir(f.absoluteFilePath()); dir.removeRecursively(); }
        }
    }
    PROGRESS;
    MODAL->show();
    m_oCreateISO = QString("mkisofs -o %1 -r -R -J -joliet-long -no-pad -no-iso-translate ").arg(m_oISOOutput);
    m_oMkisofs = QString("genisoimage -r -R -J -joliet-long -no-pad -no-iso-translate ");
    m_oMkisofs.append(QString("-A \"%1\" ").arg(_("KylinBurner")));
    m_oCreateISO.append(QString("-A \"%1\" ").arg(_("KylinBurner")));
    m_oArguments << "-c";
    if (cdrom->mediumType().startsWith("CD"))
    {
        m_oCdrecord = QString("cdrecord -v driveropts=burnfree -nopad ");
        setValue(++m_iProgress, m_oIcon, _("Prepare to burn data to CD."));
    }
    else
    {
        m_oGrowisofs = QString("growisofs -J -r -R -joliet-long -v -gui ");
        m_oGrowisofs.append(QString("-A \"%1\" ").arg(_("KylinBurner")));
        setValue(++m_iProgress, m_oIcon, _("Prepare to burn data to DVD."));
    }
    if (!sysid.isEmpty())
    {
        m_oMkisofs.append(QString("--sysid \"%1\" ").arg(sysid));
        m_oGrowisofs.append(QString("--sysid \"%1\" ").arg(sysid));
    }
    return m_oKeep;
}

bool ProcessBurnData::config()
{
    DataCDROM *cdrom;

    cdrom = (DataCDROM *)m_oCDROM;

    if (cdrom->mediumStatus() == DataMedium::MEDIUM_EMPTY)
    {
        m_oMkisofs.append(QString("-p \"%1\" ").arg(cdrom->settingPreparer()));
        m_oCreateISO.append(QString("-p \"%1\" ").arg(cdrom->settingPreparer()));
        m_oGrowisofs.append(QString("-p \"%1\" ").arg(cdrom->settingPreparer()));
        m_oMkisofs.append(QString("-P \"%1\" ").arg(cdrom->settingPublisher()));
        m_oCreateISO.append(QString("-P \"%1\" ").arg(cdrom->settingPublisher()));
        m_oGrowisofs.append(QString("-P \"%1\" ").arg(cdrom->settingPublisher()));
        m_oMkisofs.append(QString("-P \"%1\" ").arg(cdrom->settingPublisher()));
        m_oCreateISO.append(QString("-P \"%1\" ").arg(cdrom->settingPublisher()));
        //m_oGrowisofs.append(QString("-P %1 ").arg(cdrom->settingPublisher()));
        m_oMkisofs.append(QString("-copyright \"%1\" ").arg(cdrom->settingCopyright()));
        m_oCreateISO.append(QString("-copyright \"%1\" ").arg(cdrom->settingCopyright()));
        m_oGrowisofs.append(QString("-copyright \"%1\" ").arg(cdrom->settingCopyright()));
    }
    m_oMkisofs.append(QString("-V \"%1\" ").arg(cdrom->settingVolumnID()));
    m_oCreateISO.append(QString("-V \"%1\" ").arg(cdrom->settingVolumnID()));
    m_oGrowisofs.append(QString("-V \"%1\" ").arg(cdrom->settingVolumnID()));
    if (cdrom->settingSpeed())
    {
        m_oMkisofs.append(QString("-speed=%1 ").arg(cdrom->settingSpeed()));
        m_oGrowisofs.append(QString("-speed=%1 ").arg(cdrom->settingSpeed()));
        m_oCdrecord.append(QString("speed=%1 ").arg(cdrom->settingSpeed()));
    }
    m_oCdrecord.append("-eject ");
    if (!cdrom->settingCloseSession())
    {
        m_oCdrecord.append("-multi ");
    }
    else
        m_oGrowisofs.append("-dvd-compat ");

    setValue(++m_iProgress, m_oIcon, _("Prepare to start burn data."));
    return m_oKeep;
}

bool ProcessBurnData::working()
{
    DataCDROM *cdrom;
    DataDirectory *root;

    cdrom = (DataCDROM *)m_oCDROM;
    root = (DataDirectory *)m_oRoot;

    if (root->size() > 2UL * 1024UL * 1024UL * 1024UL)
    {
        debug(_("%1 data size more than 2 GB, need allow-limited-size "
                "extension").arg(_("CreateISO")));
        m_oMkisofs.append("-allow-limited-size ");
        m_oCreateISO.append("-allow-limited-size ");
        m_oGrowisofs.append("-allow-limited-size ");
    }
    if (root->size() > 4UL * 1024UL * 1024UL * 1024UL)
    {
        m_oMkisofs.append("-udf ");
        m_oCreateISO.append("-udf ");
        m_oGrowisofs.append("-udf ");
    }

    m_iCopied = 0;
    connect(root, SIGNAL(copied(QString,QString)), this, SLOT(slotsCopy(QString,QString)));
    root->copy();
    m_oTimer->start(500);
    if (m_bCancel) return false;
    if (m_bCreateISO)
    {
        m_oCreateISO.append(root->destination());
        m_oArguments << m_oCreateISO;
        m_oProcess->setArguments(m_oArguments);
        m_oProcess->start();
        while (m_oProcess->state() != QProcess::NotRunning)
            QCoreApplication::processEvents();
        m_bCreateISO = false;
    }
    if (m_bCancel) return false;
    m_oArguments.clear();
    m_oArguments << "-c";
    if (cdrom->mediumType().startsWith("CD"))
    {
        if (m_bAppendBurn)
        {
            cdrom->umount();
            m_oMSINFO = QString("cdrecord -msinfo dev=%1").arg(cdrom->device());
            m_oArguments << m_oMSINFO;
            m_oProcess->setArguments(m_oArguments);
            m_oProcess->start();
            while (m_oProcess->state() != QProcess::NotRunning)
                QCoreApplication::processEvents();
            m_bAppendBurn = false;
            m_oArguments.clear();
            m_oArguments << "-c";
            m_oMkisofs.append(QString("-C \"%1\"").arg(m_oMSINFO));
            m_oMkisofs.append(" ");
            m_oMkisofs.append(QString("-M \"%1\"").arg(cdrom->device()));
            m_oMkisofs.append(" ");
        }
        m_oMkisofs.append(root->destination());
        m_oMkisofs.append(" ");
        m_oCdrecord.append(QString("dev=%1 -").arg(cdrom->device()));
        m_oArguments << QString("%1 | %2").arg(m_oMkisofs).arg(m_oCdrecord);
    }
    else
    {
        if (m_bAppendBurn)
            m_oGrowisofs.append(QString("-M %1 %2").arg(cdrom->device()).arg(root->destination()));
        else
            m_oGrowisofs.append(QString("-Z %1 %2").arg(cdrom->device()).arg(root->destination()));
        m_oArguments << m_oGrowisofs;
    }
    m_oProcess->setArguments(m_oArguments);
    info(_("Argument setted for %1:").arg(m_oProcess->program()));
    for (int i = 0; i < m_oArguments.size(); ++i)
    {
        debug(_("Arg.%1 - %2").arg(i + 1).arg(m_oArguments.at(i)));
    }
    m_oProcess->start();
    while (m_oProcess->state() != QProcess::NotRunning)
        QCoreApplication::processEvents();
    return m_oKeep;
}

bool ProcessBurnData::worked()
{
    if (m_bCancel) return false;
    if (QProcess::NormalExit != m_oProcess->exitStatus()) return false;
    return m_oKeep;
}

void ProcessBurnData::slotsStart()
{
    cancel(false);
}

void ProcessBurnData::slotsOutput()
{
    qreal si;
    qreal t;
    qreal basic;
    QString output;
    QStringList s;

    output = m_oProcess->readAllStandardOutput();
    if (m_bCancel)
        m_oProcess->kill();
    if (m_bAppendBurn)
    {
        m_oMSINFO.clear();
        m_oMSINFO = output.trimmed();
    }

    basic = 99 - m_iProgress;

    if (output.contains("Performing OPC"))
    {
        setValue(++m_iProgress, m_oIcon, _("Performing OPC"));
        cancel(false);
    }
    if (output.startsWith("\rTrack"))
    {
        s = output.split(" ");
        for (int i = 0; i < s.size(); ++i)
        {
            if (s.at(i).isEmpty())
            {
                s.takeAt(i);
                --i;
            }
        }
        si = s.at(2).toDouble();
        t = s.at(4).toDouble();
        if (s.size() > 5)
        {
            setValue(m_iProgress + si / t * basic, m_oIcon,
                     _("Writing track %1 speed %2 written %3 %4.")
                     .arg(s.at(1)).arg(s.last()).arg(s.at(2)).arg(s.at(3)));
        }
    }

    if (output.contains("Fixating"))
        setValue(99, m_oIcon, _("Fixating(Will wait minutes)"));
}

void ProcessBurnData::slotsError()
{
    qreal v;
    int basic;
    QString output;
    QStringList ss;

    output = m_oProcess->readAllStandardError();
    if (output.contains("OPC failed."))
        m_oKeep = false;
    if (output.contains("Input/output error"))
        m_oKeep = false;
    if (output.contains(":-("))
        m_oKeep = false;
    if (output.contains("FATAL:"))
        m_oKeep = false;
    if (output.contains("error"))
        m_oKeep = false;

    if (m_bCreateISO) basic = 60 - m_iProgress;
    else basic = 99 - m_iProgress;

    if (output.contains("done, estimate"))
    {
        output = output.left(output.indexOf('%'));
        v = output.toDouble();
        v = (v / 100.00) * basic;
        m_iProgress += v;
        if (m_iProgress > 99) m_iProgress = 99;
        if (m_bCreateISO)
            setValue(m_iProgress, m_oIcon, _("Building ISO %1").arg(m_oISOOutput));
        else
        {
            if (m_fSpeed > 0)
                setValue(m_iProgress, m_oIcon, _("Burning by speed %1x").arg(m_fSpeed));
        }
    }
    if (output.contains("Current Write Speed"))
    {
        ss = output.split(" ");
        ss = ss.last().split("x");
        m_fSpeed = ss.first().toDouble();
    }

    if (output.contains("flushing cache"))
    {
        setValue(99, m_oIcon, _("flushing cache"));
    }
    if (output.contains("closing track"))
    {
        setValue(99, m_oIcon, _("closing track"));
    }
    if (output.contains("closing session"))
    {
        setValue(99, m_oIcon, _("closing session"));
    }
    error(output);
}

void ProcessBurnData::slotsFinished(int code, QProcess::ExitStatus status)
{
    DataCDROM *cdrom;

    cdrom = (DataCDROM *)m_oCDROM;

    debug(_("Exit code is %1, status : %2").arg(code).arg(status));
    if (m_bAppendBurn && cdrom->mediumType().contains("CD")) return;
    if (!m_bCreateISO) setValue(99, m_oIcon, _("Burn Data"));
    if (QProcess::NormalExit == status)
    {
        if (!m_bCreateISO)
        {
            setValue(100, m_oIcon, _("Burn Data"));
            cdrom->eject(false);
        }
    }
    cdrom->eject();
}

void ProcessBurnData::slotsCopy(QString src, QString des)
{
    qreal v;
    int basic;
    DataDirectory *root;

    root = (DataDirectory *)(m_oRoot);

    if (m_bCreateISO) basic = 30 - m_iProgress;
    else basic = 50 - m_iProgress;
    ++m_iCopied;
    v = ((m_iCopied * 1.0) / (root->total() * 1.0));
    v = v * basic;
    m_iProgress += v;
    setValue(m_iProgress, m_oIcon, _("Copied %1 to %2").arg(src).arg(des));
    debug(_("Copied %1 to %2").arg(src).arg(des));
}

ProcessBurnData::~ProcessBurnData()
{}
