#include "processearse.h"
#include "src/frame/logger.h"
#include "src/po/translation.h"
#include "src/data/datacdrom.h"
#include "src/view/modaldialog.h"

#include <QFileInfo>

ProcessEarse::ProcessEarse(void *cdrom, QObject *parent) :
    ProcessBasic(parent),
    m_oPro(new QTimer(this))
{
    m_oCDROM = cdrom;
    m_iValue = 0;
    LOGGER->registerName(this, _("PROCESS_EARSE"));
    MODAL->show();
    title(_("Earsing"));
    m_oProcessName = _("Earse");
    setRange(0, 100);
    setValue(0);
    connect(m_oPro, SIGNAL(timeout()), this, SLOT(slotsProgress()));
    m_oPro->setInterval(1000);
}

bool ProcessEarse::prepare()
{
    DataCDROM *cdrom;
    QFileInfo f;

    cdrom = (DataCDROM *)m_oCDROM;

    PROGRESS;
    MODAL->show();
    connect(this, SIGNAL(cancel(bool)), MODAL, SLOT(cancelEnable(bool)));

    m_oTimer->start(500);

    if (cdrom->mediumType().contains("CD"))
        f.setFile("/bin/cdrecord");
    else
        f.setFile("/bin/dvd+rw-format");
    if (!f.exists() || !f.isExecutable())
    {
        error(_("There is no %1 program or %1 cannot execute."
                "Please check the mkisofs is exists or not, and give "
                "the right permissions.%2 exiting...")
              .arg(f.absoluteFilePath())
              .arg("KylinBurner"));
        return false;
    }
    m_oProcess->setProgram(f.fileName());
    return true;
}

bool ProcessEarse::config()
{
    DataCDROM *cdrom;

    cdrom = (DataCDROM *)m_oCDROM;
    if (cdrom->mediumType().contains("CD"))
    {
        cdrom->umount();
        m_oArguments << "blank=fast"
                     << "driveropts=burnfree"
                     << QString("dev=%1").arg(cdrom->device())
                     << "-v"
                     << "-eject";
        setValue(m_iValue++, m_oIcon, _("Umount CD, and prepare to blank CD."));
    }
    else
    {
        m_oArguments << cdrom->device();
        m_oArguments << "-force";
    }
    return m_oKeep;
}

bool ProcessEarse::worked()
{
    DataCDROM *cdrom;

    cdrom = (DataCDROM *)m_oCDROM;

    cdrom->eject();
    return m_oKeep;
}

void ProcessEarse::slotsOutput()
{
    QString output;

    output = m_oProcess->readAllStandardOutput();

    if (output.contains("Performing OPC"))
    {
        cancel(false);
        setValue(m_iValue++, m_oIcon, _("Performing OPC"));
    }
    setValue(m_iValue++, m_oIcon, _("Blanking..."));
    m_oPro->start();
    if (output.contains("Fixating"))
        setValue(99, m_oIcon, _("Fixating(Will wait minutes)"));
}

void ProcessEarse::slotsError()
{
    int v;
    QString error;
    QStringList ss;

    error = m_oProcess->readAllStandardError();
    v = 0;
    if (error.contains("%"))
    {
        cancel(false);
        ss = error.split("\b");
        ss = ss.last().split("%");
        v = ss.first().toDouble();
        if (v < 99)
            setValue(v, m_oIcon, _("Blanking..."));
    }
    else if (error.contains(":-("))
    {
        m_oKeep = false;
    }
    else
    {
        this->error(error);
    }

}

void ProcessEarse::slotsFinished(int code, QProcess::ExitStatus status)
{
    DataCDROM *cdrom;

    cdrom = (DataCDROM *)m_oCDROM;
    if (m_oPro->isActive()) m_oPro->stop();

    debug(_("Exit code is %1, status : %2").arg(code).arg(status));
    setValue(99, m_oIcon, _("earsing...."));
    if (QProcess::NormalExit == status)
    {
        setValue(100, m_oIcon, _("Burn Data"));
        cdrom->eject();
        cdrom->eject(false);
        success(m_oProcessName);
    }
}

void ProcessEarse::slotsProgress()
{
    if (++m_iValue < 99)
        setValue(m_iValue, m_oIcon, _("Blanking..."));
}

ProcessEarse::~ProcessEarse()
{}
