#include "processmd5sum.h"
#include "src/frame/logger.h"
#include "src/data/datacdrom.h"
#include "src/po/translation.h"
#include "src/view/modaldialog.h"

#include <QDir>
#include <QFileInfo>
#include <QCoreApplication>

ProcessMd5sum::ProcessMd5sum(void *cdrom, QString checkFile, QObject *parent) :
    ProcessBasic(parent),
    m_oCDROM(cdrom),
    m_oCheckFile(checkFile),
    m_iProgress(0)
{
    m_oProcessName = _("Disk Check");
    LOGGER->registerName(this, _("PROCESS_DC"));
    title(_("Checking disk"));
}

bool ProcessMd5sum::prepare()
{
    DataCDROM *cdrom;

    PROGRESS;
    MODAL->show();

    cdrom = static_cast<DataCDROM *>(m_oCDROM);
    if (cdrom->mediumMountPoint().isEmpty()) return false;
    QDir::setCurrent(cdrom->mediumMountPoint());
    m_oProcess->setProgram("md5sum");
    m_oArguments << "-c";
    m_oArguments << m_oCheckFile;
    m_oProcess->setArguments(m_oArguments);
    setValue(++m_iProgress, m_oIcon, _("Checking..."));
    return true;
}
#if 1
bool ProcessMd5sum::worked()
{
    m_oProcess->start();
    while (!m_oProcess->atEnd())
        QCoreApplication::processEvents();
    m_oProcess->finished(0);
    return m_oKeep;
}
#endif

void ProcessMd5sum::slotsOutput()
{
    QString output;
    QStringList ss;

    output = m_oProcess->readAllStandardOutput();
    ss = output.split("\n");
    setValue(++m_iProgress, m_oIcon, _("Checking..."));
    if (output.contains(_("Fail")) || output.contains(_("FAILED")))
    {
        m_oKeep = false;
        m_oProcess->kill();
    }
}

void ProcessMd5sum::slotsError()
{
    QString error;
    QStringList ss;
    m_oKeep = false;
    error = m_oProcess->readAllStandardError();
    ss = error.split("\n");
    this->error(_("Error happened when checking disk,content: %1").arg(error));
}

void ProcessMd5sum::slotsFinished(int errCode, QProcess::ExitStatus status)
{
    if (m_oKeep)
        success(m_oProcessName);
    else
        fail(m_oProcessName);
}

ProcessMd5sum::~ProcessMd5sum()
{}
